require 'digest/md5'
require 'net/http'
require 'json'
require 'cgi'


module Mixpanel
  class Client
    attr_reader :api_key, :api_secret, :base_url, :data_url, :expire_at

    def initialize options
      @api_key    = options.fetch :api_key
      @api_secret = options.fetch :api_secret
      @base_url   = options.fetch :base_url, 'https://mixpanel.com/api/2.0'
      @data_url   = options.fetch :data_url, 'https://data.mixpanel.com/api/2.0'
      @expire_at  = options.fetch :expire_at, 90 # seconds
    end

    def request endpoint, params
      resp = Net::HTTP.get request_uri(endpoint, params)
      resp.lines.map { |l| JSON.parse l }
    end

    def request_uri endpoint, params
      URI request_url(endpoint, params)
    end


  private

    def request_url endpoint, params
      api = request_api endpoint
      url = File.join api, endpoint.to_s
      sps = sign_params endpoint, params
      qs  = encode_params sps
      "#{url}?#{qs}"
    end

    def request_api endpoint
      return data_url if endpoint == 'export'
      return base_url
    end

    def request_format endpoint
      return :raw if endpoint == 'export'
      return :json
    end

    def encode_params params
      params.map { |k,v| "#{k}=#{CGI.escape(v.to_s)}" }.sort.join('&')
    end

    def sign_params endpoint, params
      nps = normalize_params endpoint, params
      jps = nps.map { |k,v| "#{k}=#{v}" }.sort.join('')
      req = jps + api_secret
      nps[:sig] = Digest::MD5.hexdigest req
      nps
    end

    def normalize_params endpoint, params
      params.merge \
        api_key: api_key,
        expire: request_expire(params),
        format: request_format(endpoint)
    end

    def request_expire params
      return params[:expire] if params[:expire]
      Time.now.to_i + expire_at
    end

  end
end